@@ -22,18 +22,28 @@
 
 #include <stdio.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtkmozembed.h>
 
+#include "egg-entryaction.h"
+#include "egg-toolbar.h"
 #include "gobject-utils.h"
 #include "gtk-utils.h"
 #include "kz-bookmarkbar.h"
 #include "kz-mozembed.h"
 #include "kz-tablabel.h"
 #include "kz-home.h"
+#include "google.h"
 #include "kazehakase.h"
+#include "intl.h"
 #include "mozilla.h"
 #include "prefs.h"
 
+enum {
+	STATUS_LINK_MESSAGE,
+	STATUS_GESTURE
+};
+
 static void     kz_window_class_init     (KzWindowClass *class);
 static void     kz_window_init           (KzWindow *kz);
 static gboolean kz_window_delete_event   (GtkWidget *widget,
@@ -42,14 +52,20 @@ static void     kz_window_destroy        (GtkObject *object);
 
 static void     kz_window_update_nav_buttons (KzWindow *kz,
 					      KzMozEmbed *kzembed);
-static GtkWidget *create_toolbar         (KzWindow *kz);
+
+/* actions */
+static void     act_close_window         (EggAction *action, KzWindow *kz);
+static void     act_close_tab            (EggAction *action, KzWindow *kz);
+static void     act_new_window           (EggAction *action, KzWindow *kz);
+static void     act_new_tab              (EggAction *action, KzWindow *kz);
+static void     act_back                 (EggAction *action, KzWindow *kz);
+static void     act_forward              (EggAction *action, KzWindow *kz);
+static void     act_stop                 (EggAction *action, KzWindow *kz);
+static void     act_reload               (EggAction *action, KzWindow *kz);
+static void     act_location_enter       (EggAction *action, KzWindow *kz);
+static void     act_search               (EggAction *action, KzWindow *kz);
 
 /* callbacks */
-static void     back_clicked_cb          (GtkButton   *button, KzWindow *kz);
-static void     stop_clicked_cb          (GtkButton   *button, KzWindow *kz);
-static void     forward_clicked_cb       (GtkButton   *button, KzWindow *kz);
-static void     reload_clicked_cb        (GtkButton   *button, KzWindow *kz);
-static void     url_activate_cb          (GtkEntry    *widget, KzWindow *kz);
 static void     notebook_switch_page_cb  (GtkNotebook *notebook,
 					  GtkNotebookPage *page,
 					  guint page_num, KzWindow *kz);
@@ -64,7 +80,7 @@ static void     visibility_cb            (GtkMozEmbed *embed,
 					  KzWindow *kz);
 static void     load_started_cb          (GtkMozEmbed *embed, KzWindow *kz);
 static void     load_finished_cb         (GtkMozEmbed *embed, KzWindow *kz);
-static void     new_window_cb            (GtkMozEmbed *embed,
+static void     moz_new_window_cb        (GtkMozEmbed *embed,
 					  GtkMozEmbed **newEmbed,
 					  guint chromemask,
 					  KzWindow *kz);
@@ -73,9 +89,9 @@ static gint     dom_mouse_click_cb       (GtkMozEmbed *embed, gpointer event,
 					  KzWindow *kz);
 static gint     dom_mouse_dbl_click_cb   (GtkMozEmbed *embed, gpointer event,
 					  KzWindow *kz);
-static gint     dom_mouse_down_cb       (GtkMozEmbed *embed,
-					 gpointer event,
-					 KzWindow *kz);
+static gint     dom_mouse_down_cb        (GtkMozEmbed *embed,
+					  gpointer event,
+					  KzWindow *kz);
 
 /*
  * mozilla doesn't accept these signals, so we connect these funtions to
@@ -88,28 +104,45 @@ static gboolean button_release_event_cb (GtkWidget *widget,
 					 GdkEventButton *event,
 					 KzMozEmbed *kzembed);
 
+
 #warning FIXME!!
-/* action */
-static void kz_window_go_back                (KzWindow *kz);
-static void kz_window_go_forward             (KzWindow *kz);
-static void kz_window_close_current_tab_idle (KzWindow *kz);
-static void kz_window_new_reload             (KzWindow *kz);
-/*static void kz_window_home                   (KzWindow *kz);*/
-KzGestureMotion gesture_go_back[]    = { KZ_GESTURE_MOTION_LEFT,  KZ_GESTURE_MOTION_TERMINATOR };
-KzGestureMotion gesture_go_forward[] = { KZ_GESTURE_MOTION_RIGHT, KZ_GESTURE_MOTION_TERMINATOR };
-KzGestureMotion gesture_close_tab[]  = { KZ_GESTURE_MOTION_UP,    KZ_GESTURE_MOTION_TERMINATOR };
-KzGestureMotion gesture_new_reload[] = { KZ_GESTURE_MOTION_DOWN,  KZ_GESTURE_MOTION_TERMINATOR };
-KzGestureMotion gesture_home[]       = { KZ_GESTURE_MOTION_DOWN,  KZ_GESTURE_MOTION_UP, KZ_GESTURE_MOTION_TERMINATOR };
-static KzGestureDefTable gesture_table[] = {
-	{"Back",      gesture_go_back,    (GFunc) kz_window_go_back,                NULL},
-	{"Forward",   gesture_go_forward, (GFunc) kz_window_go_forward,             NULL},
-	{"Close tab", gesture_close_tab,  (GFunc) kz_window_close_current_tab_idle, NULL},
-	{"Reload",    gesture_new_reload, (GFunc) kz_window_new_reload,             NULL},
-/*	{"Home",      gesture_home,       (GFunc) kz_window_home,                   NULL},*/
+#if 1
+/* gesture sequnce */
+KzGestureMotion gesture_go_back[]    = {KZ_GESTURE_MOTION_LEFT,  KZ_GESTURE_MOTION_TERMINATOR};
+KzGestureMotion gesture_go_forward[] = {KZ_GESTURE_MOTION_RIGHT, KZ_GESTURE_MOTION_TERMINATOR};
+KzGestureMotion gesture_close_tab[]  = {KZ_GESTURE_MOTION_UP,    KZ_GESTURE_MOTION_TERMINATOR};
+KzGestureMotion gesture_reload[]     = {KZ_GESTURE_MOTION_DOWN,  KZ_GESTURE_MOTION_TERMINATOR};
+KzGestureMotion gesture_home[]       = {KZ_GESTURE_MOTION_DOWN,  KZ_GESTURE_MOTION_UP, KZ_GESTURE_MOTION_TERMINATOR};
+#endif
+
+
+#define CTRL "<control>"
+#define SHFT "<shift>"
+#define SFCT "<shift><control>"
+static EggActionGroupEntry action_entries[] = {
+  {"StockFileMenu", N_("_File"), NULL, NULL, NULL, NULL, NULL},
+  {"StockEditMenu", N_("_Edit"), NULL, NULL, NULL, NULL, NULL},
+  {"StockViewMenu", N_("_View"), NULL, NULL, NULL, NULL, NULL},
+  {"StockGoMenu",   N_("_Go"),   NULL, NULL, NULL, NULL, NULL},
+  {"StockHelpMenu", N_("_Help"), NULL, NULL, NULL, NULL, NULL},
+  {"NewWindow",     N_("_New Window"),    GTK_STOCK_NEW,        CTRL"N",  NULL, G_CALLBACK(act_new_window),   NULL},
+  {"NewTab",        N_("New _Tab"),       GTK_STOCK_NEW,        CTRL"T",  NULL, G_CALLBACK(act_new_tab),      NULL},
+  {"CloseWindow",   N_("_Close Window"),  GTK_STOCK_CLOSE,      CTRL"W",  NULL, G_CALLBACK(act_close_window), NULL},
+  {"CloseTab",      N_("Close Ta_b"),     GTK_STOCK_CLOSE,      SFCT"W",  NULL, G_CALLBACK(act_close_tab),    NULL},
+  {"Back",          N_("_Back"),          GTK_STOCK_GO_BACK,    SFCT"B",  NULL, G_CALLBACK(act_back),         NULL},
+  {"Forward",       N_("_Forward"),       GTK_STOCK_GO_FORWARD, SFCT"F",  NULL, G_CALLBACK(act_forward),      NULL},
+  {"Reload",        N_("_Reload"),        GTK_STOCK_REFRESH,    SFCT"R",  NULL, G_CALLBACK(act_reload),       NULL},
+  {"Stop",          N_("_Stop"),          GTK_STOCK_STOP,       "Escape", NULL, G_CALLBACK(act_stop),         NULL},
+  {"LocationEntry", N_("Location Entry"), GTK_STOCK_NEW,        NULL,     NULL, G_CALLBACK(act_location_enter), NULL, ENTRY_ACTION},
+  {"Search",        N_("Search"),         NULL,                 CTRL"F",  NULL, G_CALLBACK(act_search),       NULL}
 };
-static gint gesture_table_rows = sizeof(gesture_table) / sizeof(KzGestureDefTable);
+#undef CTRL
+#undef SHFT
+#undef SFCT
+
 
 static GtkWindowClass *parent_class = NULL;
+static GList *window_list = NULL;
 
 KZ_OBJECT_GET_TYPE(kz_window, "KzWindow", KzWindow,
 		   kz_window_class_init, kz_window_init,
@@ -133,102 +166,131 @@ kz_window_class_init (KzWindowClass *class)
 	widget_class->delete_event = kz_window_delete_event;
 }
 
+static void
+add_widget (EggMenuMerge *merge, GtkWidget *widget, GtkBox *box)
+{
+	gtk_box_pack_start (box, widget, FALSE, FALSE, 0);
+	gtk_widget_show (widget);
+}
+
+void
+kz_window_update_gesture (KzWindow *kz)
+{
+#warning FIXME
+	if (!kz->gesture_items) {
+		EggAction *action;
+
+		kz->gesture_items = kz_gesture_items_new();
+
+		action = egg_action_group_get_action(kz->action_group, "Back");
+		kz_gesture_items_add_action(kz->gesture_items, action,
+					    0, gesture_go_back);
+
+		action = egg_action_group_get_action(kz->action_group, "Forward");
+		kz_gesture_items_add_action(kz->gesture_items, action,
+					    0, gesture_go_forward);
+
+		action = egg_action_group_get_action(kz->action_group, "CloseTab");
+		kz_gesture_items_add_action(kz->gesture_items, action,
+					    0, gesture_close_tab);
+
+		action = egg_action_group_get_action(kz->action_group, "Reload");
+		kz_gesture_items_add_action(kz->gesture_items, action,
+					    0, gesture_reload);
+	} else {
+	}
+}
+
 static void
 kz_window_init (KzWindow *kz)
 {
-	kz->topLevelVBox       = NULL;
-	kz->menuBar            = NULL;
-	kz->fileMenuItem       = NULL;
-	kz->fileMenu           = NULL;
-	kz->fileClose          = NULL;
-	kz->fileQuit           = NULL;
-	kz->toolbarHBox        = NULL;
-	kz->toolbar            = NULL;
-	kz->backButton         = NULL;
-	kz->stopButton         = NULL;
-	kz->forwardButton      = NULL;
-	kz->reloadButton       = NULL;
-	kz->homeButton         = NULL;
+	GtkWidget *menu_box, *toolbar;
+	gint i;
+	GError *err;
+
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	kz->topLevelVBox       = gtk_vbox_new(FALSE, 0);
+
         kz->urlEntry           = NULL;
 	kz->progressAreaHBox   = NULL;
         kz->progressBar        = NULL;
-	kz->statusAlign        = NULL;
-	kz->statusBar          = NULL;
-	kz->notebook           = NULL;
-	kz->findAreaHBox       = NULL;
-	kz->findArea           = NULL;
-	kz->kztoolBar          = NULL; /* Kazehakase tool bar */
+	kz->statusAlign        = gtk_alignment_new(0, 0, 1, 1);
+	kz->statusBar          = gtk_statusbar_new();
+	kz->notebook           = gtk_notebook_new();
+	kz->findAreaHBox       = gtk_hbox_new(FALSE, 0);
+	kz->findArea           = gtk_entry_new();
+	kz->kztoolBar          = GTK_WIDGET(kz_bookmark_bar_new(kz));
+
+	kz->accel_group        = gtk_accel_group_new();
+	kz->action_group       = egg_action_group_new("KzWindow");;
+	kz->menu_merge         = egg_menu_merge_new();
+	kz->gesture_items      = NULL;
 
 	kz->statusMessage      = NULL;
 	kz->tempMessage        = NULL;
 
-	kz->menuBarOn          = TRUE;
-	kz->toolBarOn          = TRUE;
-	kz->locationBarOn      = TRUE;
-	kz->statusBarOn        = TRUE;
 	kz->didFind            = FALSE; /* for find keyword */
-}
 
-GtkWidget *
-kz_window_new(const gchar *url)
-{
-	KzWindow *kz = KZ_WINDOW(g_object_new(KZ_TYPE_WINDOW, NULL));
-	GtkWidget *hbox, *toolbar;
+	/* actions and gesture definition */
+	egg_action_group_set_accel_group(kz->action_group, kz->accel_group);
+	for (i = 0; i < G_N_ELEMENTS(action_entries); i++)
+		action_entries[i].user_data = kz;
+	egg_action_group_add_actions (kz->action_group,
+				      action_entries,
+				      G_N_ELEMENTS(action_entries));
+	for (i = 0; i < G_N_ELEMENTS(action_entries); i++)
+		action_entries[i].user_data = NULL;
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	kz_window_update_gesture(kz);
 
-	GTK_WINDOW(kz)->type = GTK_WINDOW_TOPLEVEL;
-
-	/* new vbox */
-	kz->topLevelVBox = gtk_vbox_new(FALSE, 0);
+	/* top level vbox */
 	gtk_container_add(GTK_CONTAINER(kz),
 			  kz->topLevelVBox);
 	gtk_widget_show(kz->topLevelVBox);
 
-	/* new horiz toolbar with buttons + icons */
-	kz->toolbar = create_toolbar (kz);
-	gtk_box_pack_start(GTK_BOX(hbox), kz->toolbar,
+	/* menu & toolbar */
+	menu_box = gtk_vbox_new (FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(kz->topLevelVBox), menu_box,
 			   FALSE,
 			   FALSE,
 			   0);   
-	gtk_widget_show(kz->toolbar);
+	gtk_widget_show(menu_box);
 
-	/* create the url text entry */
-	kz->urlEntry = gtk_combo_new();
-	gtk_combo_disable_activate(GTK_COMBO(kz->urlEntry));
-	toolbar = gtk_toolbar_new();
-	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar),
-				    GTK_ORIENTATION_HORIZONTAL);
-	gtk_widget_show(toolbar);
-	gtk_box_pack_start(GTK_BOX(hbox), kz->urlEntry,
-			   TRUE, 
-			   TRUE, 
-			   0);
-	gtk_widget_show(kz->urlEntry);
+	egg_menu_merge_set_accel_group(kz->menu_merge, kz->accel_group);
+	egg_menu_merge_insert_action_group(kz->menu_merge, kz->action_group, 0);
+	g_signal_connect(kz->menu_merge, "add_widget",
+			 G_CALLBACK(add_widget), menu_box);
+	egg_menu_merge_add_ui_from_file(kz->menu_merge,
+					KZ_DATA_DIR"/kz-ui-menu.xml",
+					&err);
 
-	/* hook up the activate signal to the right callback */
-	g_signal_connect(G_OBJECT(GTK_COMBO(kz->urlEntry)->entry), "activate",
-			 G_CALLBACK(url_activate_cb), kz);
+	toolbar = egg_menu_merge_get_widget(kz->menu_merge, "/MainToolBar");
+	if (EGG_IS_TOOLBAR(toolbar)) {
+		EggToolItem *t = egg_tool_item_new();
+		GtkWidget *b = google_new(kz);
+		gtk_container_add(GTK_CONTAINER(t), b);
+		gtk_widget_show(b);
+		gtk_widget_show(GTK_WIDGET(t));
+		egg_toolbar_append_tool_item (EGG_TOOLBAR(toolbar),
+					      t);
+		EggToolItem *toolitem = egg_tool_item_new();
+		GtkWidget *button = kz_home_button_new(kz);
+		gtk_container_add(GTK_CONTAINER(toolitem), button);
+		gtk_widget_show(button);
+		gtk_widget_show(GTK_WIDGET(toolitem));
+		egg_toolbar_append_tool_item (EGG_TOOLBAR(toolbar),
+					      toolitem);
+	}
 
-	/* add that hbox to the vbox */
-	gtk_box_pack_start(GTK_BOX(kz->topLevelVBox), 
-			   hbox,
-			   FALSE, 
-			   FALSE, 
-			   0);    
-	gtk_widget_show(hbox);
+	gtk_window_add_accel_group(GTK_WINDOW(kz), kz->accel_group);
 
 	/* KzWindow tool bar */
-	kz->kztoolBar = GTK_WIDGET(kz_bookmark_bar_new(kz));
 	gtk_box_pack_start(GTK_BOX(kz->topLevelVBox), 
-			   kz->kztoolBar,
-			   FALSE, 
-			   FALSE, 
-			   0);    
+			   kz->kztoolBar, FALSE, FALSE, 0);    
 	gtk_widget_show(kz->kztoolBar);
 
 	/* create notebook widget */
-	kz->notebook = gtk_notebook_new();
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(kz->notebook), TRUE);
 	gtk_notebook_set_scrollable(GTK_NOTEBOOK(kz->notebook), TRUE);
 	g_signal_connect(G_OBJECT(kz->notebook), "switch_page", 
@@ -237,52 +299,66 @@ kz_window_new(const gchar *url)
 	gtk_container_add(GTK_CONTAINER(kz->topLevelVBox), kz->notebook);
 	gtk_widget_show(kz->notebook);
 
-	/* create new mozembed tab */
-	kz_window_open_new_tab(kz, url);
+	/* mozembed tab */
+	kz_window_open_new_tab(kz, NULL);
 
-	/* create the new hbox for the search area */
-	kz->findAreaHBox = gtk_hbox_new(FALSE, 0);
+	/* search area */
 	gtk_box_pack_start(GTK_BOX(kz->topLevelVBox), kz->findAreaHBox,
-			   FALSE, 
-			   FALSE, 
-			   0);   
+			   FALSE, FALSE, 0);   
 	gtk_widget_show(kz->findAreaHBox);
 
-	/* create our new progress bar */
-	kz->findArea = gtk_entry_new();
+	/* find area */
 	gtk_box_pack_start(GTK_BOX(kz->findAreaHBox), kz->findArea,
-			   FALSE,
-			   FALSE,
-			   0); 
+			   FALSE, FALSE, 0); 
 	g_signal_connect(G_OBJECT(GTK_ENTRY(kz->findArea)), "activate",
 			 G_CALLBACK(find_keyword_cb), kz);
 	gtk_widget_show(kz->findArea);
 
 	/* create our status area and the alignment object that will keep it */
 	/* from expanding */
-	kz->statusAlign = gtk_alignment_new(0, 0, 1, 1);
 	gtk_widget_set_size_request(kz->statusAlign, 1, -1);
 	gtk_widget_show(kz->statusAlign);
 	/* create the status bar */
-	kz->statusBar = gtk_statusbar_new();
 	gtk_container_add(GTK_CONTAINER(kz->statusAlign), kz->statusBar);
 	gtk_box_pack_start(GTK_BOX(kz->findAreaHBox), kz->statusAlign,
 			   TRUE, 
 			   TRUE, 
 			   0);
 	gtk_widget_show(kz->statusBar);
-	/* by default none of the buttons are marked as sensitive. */
-	gtk_widget_set_sensitive(kz->backButton, FALSE);
-/*	gtk_widget_set_sensitive(kz->stopButton, FALSE); */
-	gtk_widget_set_sensitive(kz->forwardButton, FALSE);
-/*	gtk_widget_set_sensitive(kz->reloadButton, FALSE); */
 
-	/* FIX ME! set last window size! */ 
+	KZ_WINDOW_SET_SENSITIVE(kz, "Back",    FALSE);
+	KZ_WINDOW_SET_SENSITIVE(kz, "Stop",    FALSE);
+	KZ_WINDOW_SET_SENSITIVE(kz, "Forward", FALSE);
+	KZ_WINDOW_SET_SENSITIVE(kz, "Reload",  FALSE);
+	KZ_WINDOW_SET_VISIBLE(kz, "Stop",   FALSE);
+	KZ_WINDOW_SET_VISIBLE(kz, "Reload", TRUE);
+
+	/* FIXME! set last window size! */ 
 	gtk_window_set_default_size(GTK_WINDOW(kz), 
 				    prefs.width, prefs.height);
 
 	gtk_widget_show(GTK_WIDGET(kz));
 
+	window_list = g_list_append(window_list, kz);
+}
+
+GList *
+kz_window_get_window_list()
+{
+	return window_list;
+}
+
+GtkWidget *
+kz_window_new(const gchar *url)
+{
+	KzWindow *kz = KZ_WINDOW(g_object_new(KZ_TYPE_WINDOW,
+					      "type", GTK_WINDOW_TOPLEVEL,
+					      NULL));
+	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+
+	if (GTK_IS_MOZ_EMBED(widget))
+		gtk_moz_embed_load_url(GTK_MOZ_EMBED(widget), url);
+
 	return GTK_WIDGET (kz);
 }
 
@@ -295,18 +371,17 @@ cb_gesture_stack_motion(KzGesture *gesture, KzGestureMotion motion, KzWindow *kz
 	g_return_if_fail(KZ_IS_GESTURE(gesture));
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	kz_gesture_create_gesture_string(gesture, buf1,
-					 sizeof(buf1) / sizeof(gchar));
+	kz_gesture_create_gesture_string(gesture, buf1, G_N_ELEMENTS(buf1));
 	label = kz_gesture_get_matched_label(gesture);
 	if (label)
-		g_snprintf(buf2, sizeof(buf2) / sizeof(gchar),
+		g_snprintf(buf2, G_N_ELEMENTS(buf2),
 			   "Gesture: %s(Action: %s)", buf1, label);
 	else
-		g_snprintf(buf2, sizeof(buf2) / sizeof(gchar),
+		g_snprintf(buf2, G_N_ELEMENTS(buf2),
 			   "Gesture: %s", buf1);
 
-
-	gtk_statusbar_push(GTK_STATUSBAR(kz->statusBar), 1, buf2);
+	gtk_statusbar_push(GTK_STATUSBAR(kz->statusBar),
+			   STATUS_GESTURE, buf2);
 }
 
 GtkWidget *
@@ -314,26 +389,32 @@ kz_window_open_new_tab (KzWindow *kz, const gchar *url)
 {
 	KzMozEmbed *kzembed;
 	KzTabLabel *kztab;
-	KzGestureDefTable *table;
-	gint i;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 
 	kzembed = KZ_MOZ_EMBED(kz_moz_embed_new(kz, url));
 	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
 
-#if 1 /* FIXME */
-	table = g_memdup(gesture_table, sizeof(gesture_table));
-	for (i = 0; i < gesture_table_rows; i++)
-	{
-		table[i].data = kz;
-	}
-	kzembed->gesture->table      = table;
-	kzembed->gesture->table_rows = gesture_table_rows;
+	kz_gesture_set_items(kzembed->gesture, kz->gesture_items);
 	g_signal_connect(G_OBJECT(kzembed->gesture), "stack_motion",
 			 G_CALLBACK(cb_gesture_stack_motion), kz);
-#endif /* FIXME */
 
+	/*
+	 * mouse event signal
+	 */
+	g_signal_connect(G_OBJECT(kzembed),
+			 "dom_mouse_down",
+			 G_CALLBACK(dom_mouse_down_cb), kz);
+	g_signal_connect(G_OBJECT(kz),
+			 "motion_notify_event",
+			 G_CALLBACK(motion_notify_event_cb), kzembed);
+	g_signal_connect(G_OBJECT(kz),
+			 "button_release_event",
+			 G_CALLBACK(button_release_event_cb), kzembed);
+
+	/*
+	 * set callbacks
+	 */
 	g_signal_connect(G_OBJECT(kzembed), "title",
 			 G_CALLBACK(title_changed_cb), kz);
 	g_signal_connect(G_OBJECT(kzembed), "location",
@@ -345,7 +426,7 @@ kz_window_open_new_tab (KzWindow *kz, const gchar *url)
 	g_signal_connect(G_OBJECT(kzembed), "net_stop",
 			 G_CALLBACK(load_finished_cb), kz);
 	g_signal_connect(G_OBJECT(kzembed), "new_window",
-			 G_CALLBACK(new_window_cb), kz);
+			 G_CALLBACK(moz_new_window_cb), kz);
 	g_signal_connect(G_OBJECT(kzembed), "dom_mouse_click",
 			 G_CALLBACK(dom_mouse_click_cb), kz);
 	g_signal_connect(G_OBJECT(kzembed), "dom_mouse_dbl_click",
@@ -353,18 +434,6 @@ kz_window_open_new_tab (KzWindow *kz, const gchar *url)
 	g_signal_connect(G_OBJECT(kzembed), "destroy",
 			 G_CALLBACK(close_tab_cb), kz);
 
-	/* mouse event signal */
-	g_signal_connect(G_OBJECT(kzembed),
-			 "dom_mouse_down",
-			 G_CALLBACK(dom_mouse_down_cb), kz);
-	/* FIXME!!: these callback should be disconnected when kzembed is destroyed */
-	g_signal_connect(G_OBJECT(kz),
-			 "motion_notify_event",
-			 G_CALLBACK(motion_notify_event_cb), kzembed);
-	g_signal_connect(G_OBJECT(kz),
-			 "button_release_event",
-			 G_CALLBACK(button_release_event_cb), kzembed);
-
 	gtk_widget_show(GTK_WIDGET(kzembed));
 	gtk_widget_show(GTK_WIDGET(kztab));
 
@@ -376,31 +445,52 @@ kz_window_open_new_tab (KzWindow *kz, const gchar *url)
 }
 
 void
-kz_window_close_tab (KzWindow *kz, KzMozEmbed *kzembed)
+kz_window_close_tab (KzWindow *kz, GtkWidget *widget)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
+	g_return_if_fail(GTK_IS_WIDGET(widget));
 
-	g_signal_handlers_disconnect_by_func(kz, motion_notify_event_cb, kzembed);
-	g_signal_handlers_disconnect_by_func(kz, button_release_event_cb, kzembed);
+	if (KZ_IS_MOZ_EMBED(widget)) {
+		g_signal_handlers_disconnect_by_func(kz,
+						     motion_notify_event_cb,
+						     widget);
+		g_signal_handlers_disconnect_by_func(kz,
+						     button_release_event_cb,
+						     widget);
+	}
 
-	gtk_widget_destroy(GTK_WIDGET(kzembed));
+	gtk_widget_destroy(widget);
 }
 
+typedef struct _CloseTab
+{
+	KzWindow  *kz;
+	GtkWidget *widget;
+} CloseTab;
+
 static gboolean
 idle_close_tab (gpointer data)
 {
-	KzMozEmbed *kzembed = data;
-	kz_window_close_tab(kzembed->parent_kazehakase, kzembed);
+	CloseTab *close_tab = data;
+	kz_window_close_tab(close_tab->kz, close_tab->widget);
+	g_free(close_tab);
+
 	return FALSE;
 }
 
 void
-kz_window_close_tab_idle (KzWindow *kz, KzMozEmbed *kzembed)
+kz_window_close_tab_idle (KzWindow *kz, GtkWidget *widget)
 {
+	CloseTab *close_tab;
+
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
-	gtk_idle_add(idle_close_tab, kzembed);
+	g_return_if_fail(GTK_IS_WIDGET(widget));
+
+	close_tab = g_new(CloseTab, 1);
+	close_tab->kz = kz;
+	close_tab->widget = widget;
+
+	gtk_idle_add(idle_close_tab, close_tab);
 }
 
 static gboolean
@@ -428,15 +518,31 @@ kz_window_destroy (GtkObject *object)
 {
 	KzWindow *kz = KZ_WINDOW(object);
 
-	g_return_if_fail (KZ_WINDOW(kz));
+	g_return_if_fail (KZ_IS_WINDOW(kz));
+
+	window_list = g_list_remove(window_list, kz);
 
 	g_free(kz->tempMessage);
         kz->tempMessage = NULL;
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		GTK_OBJECT_CLASS (parent_class)->destroy(object);
+	if (kz->menu_merge)
+		g_object_unref(kz->menu_merge);
+	kz->menu_merge = NULL;
 
-	gtk_main_quit();
+	if (kz->action_group)
+		g_object_unref(kz->action_group);
+	kz->action_group = NULL;
+
+	if (kz->gesture_items)
+		kz_gesture_items_unref(kz->gesture_items);
+	kz->gesture_items = NULL;
+
+	if (GTK_OBJECT_CLASS (parent_class)->destroy) {
+		GTK_OBJECT_CLASS (parent_class)->destroy(object);
+	}
+
+	if (!kz_window_get_window_list())
+		gtk_main_quit();
 }
 
 static void
@@ -447,7 +553,7 @@ kz_window_update_nav_buttons (KzWindow *kz, KzMozEmbed *kzembed)
 	gboolean can_go_forward;
 	gboolean can_stop;
 
-	g_return_if_fail (kz);
+	g_return_if_fail (KZ_IS_WINDOW(kz));
 	g_return_if_fail (KZ_IS_MOZ_EMBED (kzembed));
 
 	embed = GTK_MOZ_EMBED(kzembed);
@@ -456,160 +562,166 @@ kz_window_update_nav_buttons (KzWindow *kz, KzMozEmbed *kzembed)
 	can_go_forward = gtk_moz_embed_can_go_forward(embed);
 	can_stop = kz_moz_embed_is_loading(KZ_MOZ_EMBED(embed));
 
-	gtk_widget_set_sensitive(kz->backButton, can_go_back);
-	gtk_widget_set_sensitive(kz->forwardButton, can_go_forward);
-	if (can_stop) {
-		gtk_widget_show(kz->stopButton);
-		gtk_widget_hide(kz->reloadButton);
-		gtk_widget_set_sensitive(kz->stopButton, TRUE);
-		gtk_widget_set_sensitive(kz->reloadButton, FALSE);
-	} else {
-		gtk_widget_hide(kz->stopButton);
-		gtk_widget_show(kz->reloadButton);
-		gtk_widget_set_sensitive(kz->stopButton, FALSE);
-		gtk_widget_set_sensitive(kz->reloadButton, TRUE);
-	}
-}
-
-
-static GtkWidget *
-create_toolbar (KzWindow *kz)
-{
-	GtkWidget *toolbar;
-
-	toolbar = gtk_toolbar_new();
-	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar),
-				    GTK_ORIENTATION_HORIZONTAL);
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar),
-			      GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_icon_size(GTK_TOOLBAR(toolbar),
-				  GTK_ICON_SIZE_SMALL_TOOLBAR);
-	gtk_toolbar_set_tooltips(GTK_TOOLBAR(toolbar), TRUE);
-
-	/* new back button */
-	kz->backButton =
-		gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
-					 GTK_STOCK_GO_BACK,
-					 "Go Back",
-					 "Go Back",
-					 G_CALLBACK(back_clicked_cb),
-					 kz,1);
-
-	/* new forward button */
-	kz->forwardButton =
-		gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
-					 GTK_STOCK_GO_FORWARD,
-					 "Forward",
-					 "Forward",
-					 G_CALLBACK(forward_clicked_cb),
-					 kz, 2);
-	/* new stop button */
-	kz->stopButton = 
-		gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
-					 GTK_STOCK_STOP,
-					 "Stop",
-					 "Stop",
-					 G_CALLBACK(stop_clicked_cb),
-					 kz, 3);
-	/* new reload button */
-	kz->reloadButton = 
-		gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
-					 GTK_STOCK_REFRESH,
-					 "Reload",
-					 "Reload",
-					 G_CALLBACK(reload_clicked_cb),
-					 kz, 4);
-
-	/* new home button */
-	kz->homeButton = kz_home_button_new(kz);
-	gtk_toolbar_insert_widget(GTK_TOOLBAR(toolbar),
-				  kz->homeButton,
-				 "Home",
-				 "Home",
-				  5);
-	return toolbar;
+	KZ_WINDOW_SET_SENSITIVE(kz, "Back",    can_go_back);
+	KZ_WINDOW_SET_SENSITIVE(kz, "Forward", can_go_forward);
+	KZ_WINDOW_SET_VISIBLE  (kz, "Stop",    can_stop);
+	KZ_WINDOW_SET_VISIBLE  (kz, "Reload",  !can_stop);
+	KZ_WINDOW_SET_SENSITIVE(kz, "Stop",    can_stop);
+	KZ_WINDOW_SET_SENSITIVE(kz, "Reload",  !can_stop);
 }
 
 void
 kz_window_set_visibility (KzWindow *kz, gboolean visibility)
 {
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
 	if (!visibility)
 	{
 		gtk_widget_hide(GTK_WIDGET (kz));
-		return;
 	}
-
-#if 0
-	/*
-	 * since they are on the same line here...
-	 */
-	if (kz->toolBarOn || kz->locationBarOn)
-		gtk_widget_show(kz->toolbarHBox);
-	else 
-		gtk_widget_hide_all(kz->toolbarHBox);
-#endif
-
-	if (kz->statusBarOn)
-	  gtk_widget_show(kz->findAreaHBox);
 	else
-	  gtk_widget_hide(kz->findAreaHBox);
+	{
+		gtk_widget_show(GTK_WIDGET(kz));
+	}
 }
 
 void
 kz_window_load_url (KzWindow *kz, const gchar *url)
 {
-	gtk_moz_embed_load_url(GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)), url);
+	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	if (GTK_IS_MOZ_EMBED(widget))
+		gtk_moz_embed_load_url(GTK_MOZ_EMBED(widget), url);
 }
 
 gchar *
 kz_window_get_title (KzWindow *kz)
 {
-	return mozilla_get_title(GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+
+	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
+
+	if (GTK_IS_MOZ_EMBED(widget))
+		return mozilla_get_title(GTK_MOZ_EMBED(widget));
+	else
+		return NULL;
 }
 
 gchar *
 kz_window_get_uri (KzWindow *kz)
 {
-	return gtk_moz_embed_get_location(GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+
+	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
+
+	if (GTK_IS_MOZ_EMBED(widget))
+		return gtk_moz_embed_get_location(GTK_MOZ_EMBED(widget));
+	else
+		return NULL;
 }
 
 static void
-back_clicked_cb (GtkButton *button, KzWindow *kz)
+act_close_window (EggAction *action, KzWindow *kz)
 {
-	gtk_moz_embed_go_back(GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	gtk_widget_destroy(GTK_WIDGET(kz));
+}
+
+static void
+act_close_tab (EggAction *action, KzWindow *kz)
+{
+	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+	if (widget)
+		kz_window_close_tab_idle(kz, widget);
+}
+
+static void
+act_new_window (EggAction *action, KzWindow *kz)
+{
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+	kz_window_new(NULL);
+}
+
+static void
+act_new_tab (EggAction *action, KzWindow *kz)
+{
+	kz_window_open_new_tab(kz, NULL);
+}
+
+static void
+act_back (EggAction *action, KzWindow *kz)
+{
+	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	if (GTK_IS_MOZ_EMBED(widget))
+		gtk_moz_embed_go_back(GTK_MOZ_EMBED(widget));
 }
 
 static void 
-stop_clicked_cb (GtkButton *button, KzWindow *kz)
+act_stop (EggAction *action, KzWindow *kz)
 {
-	gtk_moz_embed_stop_load(GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	if (GTK_IS_MOZ_EMBED(widget))
+		gtk_moz_embed_stop_load(GTK_MOZ_EMBED(widget));
 }
 
 static void
-forward_clicked_cb (GtkButton *button, KzWindow *kz)
+act_forward (EggAction *action, KzWindow *kz)
 {
-	gtk_moz_embed_go_forward(GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	if (GTK_IS_MOZ_EMBED(widget))
+		gtk_moz_embed_go_forward(GTK_MOZ_EMBED(widget));
 }
 
 static void
-reload_clicked_cb  (GtkButton *button, KzWindow *kz)
+act_reload  (EggAction *action, KzWindow *kz)
 {
 	GdkModifierType state = (GdkModifierType)0;
 	gint x, y;
+	GtkWidget *widget;
+
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
 	gdk_window_get_pointer(NULL, &x, &y, &state);
 
-	gtk_moz_embed_reload(GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)),
-			     (state & GDK_SHIFT_MASK) ?
-			     GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE : 
-			     GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
+	widget = KZ_WINDOW_CURRENT_PAGE(kz);
+
+	if (GTK_IS_MOZ_EMBED(widget))
+		gtk_moz_embed_reload(GTK_MOZ_EMBED(widget),
+				     (state & GDK_SHIFT_MASK) ?
+				     GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE : 
+				     GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
 }
 
 static void
-url_activate_cb    (GtkEntry *widget, KzWindow *kz)
+act_location_enter (EggAction *action, KzWindow *kz)
 {
-	const gchar *text = gtk_entry_get_text(GTK_ENTRY(widget));
+	const gchar *uri;
 
-	gtk_moz_embed_load_url(GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)), text);
+	g_return_if_fail(EGG_IS_ENTRY_ACTION(action));
+
+	uri = egg_entry_action_get_text(EGG_ENTRY_ACTION(action));
+	gtk_moz_embed_load_url(GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)), uri);
+}
+
+static void
+act_search (EggAction *action, KzWindow *kz)
+{
+	g_return_if_fail(EGG_IS_ACTION(action));
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	gtk_widget_grab_focus(kz->findArea);
 }
 
 static void
@@ -627,8 +739,12 @@ notebook_switch_page_cb (GtkNotebook *notebook, GtkNotebookPage *page,
 	}
 	if (newLocation)
 	{
-		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(kz->urlEntry)->entry),
-				   newLocation);
+		EggAction *action
+			= egg_action_group_get_action(kz->action_group,
+						      "LocationEntry");
+		if (EGG_IS_ENTRY_ACTION(action))
+			egg_entry_action_set_text(EGG_ENTRY_ACTION(action),
+						  newLocation);
 	}
 	kz_window_update_nav_buttons(kz, kzembed);
 }
@@ -639,11 +755,7 @@ find_keyword_cb(GtkWidget *widget, KzWindow *kz)
 	GtkMozEmbed *embed = GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz));
 	const gchar *text = gtk_entry_get_text(GTK_ENTRY(widget));
 
-	gtk_widget_grab_focus(GTK_BIN(embed)->child);
 	mozilla_find(embed, text, &kz->didFind);
-	g_print("find=%d, page=%s\n",
-		kz->didFind,
-		kz_moz_embed_get_title(KZ_MOZ_EMBED(embed)));
 }
 
 static void
@@ -652,8 +764,8 @@ title_changed_cb (GtkMozEmbed *embed, KzWindow *kz)
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
 	if (embed == GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz))) {
-		gchar *title = (gchar *) kz_moz_embed_get_title(KZ_MOZ_EMBED(embed));
-		gtk_window_set_title(GTK_WINDOW(kz), title);
+		const gchar *title = kz_moz_embed_get_title(KZ_MOZ_EMBED(embed));
+		gtk_window_set_title(GTK_WINDOW(kz), (gchar *) title);
 	}
 }
 
@@ -668,9 +780,15 @@ location_changed_cb (GtkMozEmbed *embed, KzWindow *kz)
 	newLocation = kz_moz_embed_get_location(KZ_MOZ_EMBED(embed));
 	if (newLocation)
 	{
-		if (embed == GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)))
-			gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(kz->urlEntry)->entry),
-					   newLocation);
+		EggAction *action
+			= egg_action_group_get_action(kz->action_group,
+						      "LocationEntry");
+		if (embed == GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz))
+		    && EGG_IS_ENTRY_ACTION(action))
+		{
+			egg_entry_action_set_text(EGG_ENTRY_ACTION(action),
+						  newLocation);
+		}
 	}
 	else
 	{
@@ -690,7 +808,8 @@ link_message_cb (GtkMozEmbed *embed, KzWindow *kz)
 
 	newMessage = mozilla_get_link_message(embed);
 	if(kz->statusBar)
-		gtk_statusbar_push(GTK_STATUSBAR(kz->statusBar), 1, newMessage);
+		gtk_statusbar_push(GTK_STATUSBAR(kz->statusBar),
+				   STATUS_LINK_MESSAGE, newMessage);
 
 #if 0
 	if (KZ_MOZ_EMBED(embed)->popup_window)
@@ -723,23 +842,15 @@ visibility_cb (GtkMozEmbed *embed, gboolean visibility, KzWindow *kz)
 static void
 load_started_cb (GtkMozEmbed *embed, KzWindow *kz)
 {
-	if (embed != GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz))) return;
-
-	gtk_widget_show(kz->stopButton);
-	gtk_widget_hide(kz->reloadButton);
-	gtk_widget_set_sensitive(kz->stopButton, TRUE);
-	gtk_widget_set_sensitive(kz->reloadButton, FALSE);
+	if (embed == GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)))
+		kz_window_update_nav_buttons(kz, KZ_MOZ_EMBED(embed));
 }
 
 static void
 load_finished_cb (GtkMozEmbed *embed, KzWindow *kz)
 {
-	if (embed != GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz))) return;
-
-	gtk_widget_hide(kz->stopButton);
-	gtk_widget_show(kz->reloadButton);
-	gtk_widget_set_sensitive(kz->stopButton, FALSE);
-	gtk_widget_set_sensitive(kz->reloadButton, TRUE);
+	if (embed == GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)))
+		kz_window_update_nav_buttons(kz, KZ_MOZ_EMBED(embed));
 }
 
 static void 
@@ -749,9 +860,9 @@ destroy_browser_cb (GtkMozEmbed *embed, GtkWidget *transient_window)
 }
 
 static void
-new_window_cb (GtkMozEmbed *embed,
-	       GtkMozEmbed **newEmbed, guint chromemask,
-	       KzWindow *kz)
+moz_new_window_cb (GtkMozEmbed *embed,
+		   GtkMozEmbed **newEmbed, guint chromemask,
+		   KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
@@ -760,12 +871,12 @@ new_window_cb (GtkMozEmbed *embed,
 		/* FIXME! this is ad hoc. */
 		GtkWidget *newWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
-		gtk_window_set_transient_for(GTK_WINDOW(newWindow), GTK_WINDOW(kz));
+		gtk_window_set_transient_for(GTK_WINDOW(newWindow),
+					     GTK_WINDOW(kz));
 		*newEmbed = GTK_MOZ_EMBED(kz_moz_embed_new(NULL, NULL));
 		g_signal_connect(G_OBJECT(*newEmbed),"destroy_browser",
 				 G_CALLBACK(destroy_browser_cb),newWindow);
 		gtk_container_add(GTK_CONTAINER(newWindow),GTK_WIDGET(*newEmbed));
-		gtk_widget_show_all(newWindow);
 	}
 	else
 	{
@@ -777,6 +888,7 @@ new_window_cb (GtkMozEmbed *embed,
 static void
 close_tab_cb (GtkObject *obj, KzWindow *kz)
 {
+#if 0
 	KzMozEmbed *kzembed;
 	gint n, n_pages;
 
@@ -797,6 +909,7 @@ close_tab_cb (GtkObject *obj, KzWindow *kz)
 	{
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(kz->notebook), n);
 	}
+#endif
 }
 
 #warning FIXME
@@ -808,7 +921,8 @@ dom_mouse_click_cb (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
      	glong type;
      	MouseEventInfo info;
 
-	g_return_val_if_fail(KZ_WINDOW(kz), FALSE);
+	g_return_val_if_fail(GTK_IS_MOZ_EMBED(embed), FALSE);
+	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
 
 	time = GDK_CURRENT_TIME;
 	
@@ -817,7 +931,9 @@ dom_mouse_click_cb (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 
 	switch (button) {
 	 case 0:
-		 if ((info.modifier & SHIFT_KEY) && (type & CONTEXT_LINK) && info.cinfo.link)
+		 if ((info.modifier & SHIFT_KEY) &&
+		     (type & CONTEXT_LINK) &&
+		     info.cinfo.link)
 		 {
 			 kz_window_open_new_tab(kz, info.cinfo.link);
 			 return TRUE;
@@ -836,8 +952,10 @@ dom_mouse_click_cb (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 static gint
 dom_mouse_dbl_click_cb (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 {
+#if 0
 	gint button;
 
+	g_return_val_if_fail(GTK_IS_MOZ_EMBED(embed), FALSE);
 	g_return_val_if_fail(KZ_WINDOW(kz), FALSE);
 
 	button = mozilla_get_button(event);
@@ -851,6 +969,7 @@ dom_mouse_dbl_click_cb (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 	}
 
 	return FALSE;
+#endif
 }
 
 static gint
@@ -861,6 +980,7 @@ dom_mouse_down_cb (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
      	glong type;
      	MouseEventInfo info;
 
+	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(embed), FALSE);
 
 	kzembed = KZ_MOZ_EMBED(embed);
@@ -880,29 +1000,33 @@ dom_mouse_down_cb (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 		gdk_window_get_position (GTK_WIDGET(kzembed)->window,
 					 &win_x_pos, &win_y_pos);
 
-		x = win_x + win_x_pos + x_pos;
-		y = win_y + win_y_pos + y_pos;
-
+		gtk_widget_get_pointer (GTK_WIDGET(kzembed),
+					&x, &y);
 		kz_gesture_start(kzembed->gesture, 0, x, y);
 
 		if ((type & CONTEXT_LINK) && info.cinfo.link)
 			kzembed->gesture_location = g_strdup(info.cinfo.link);
 
 		if (!cursor) cursor = gdk_cursor_new (GDK_HAND1);
-		gdk_pointer_grab (GTK_WIDGET(kzembed->parent_kazehakase)->window, FALSE,
+		gdk_pointer_grab (GTK_WIDGET(kzembed->parent_kazehakase)->window,
+				  FALSE,
 				  GDK_POINTER_MOTION_MASK |
 				  GDK_BUTTON_RELEASE_MASK |
 				  GDK_BUTTON_PRESS_MASK,
 				  NULL, cursor, gtk_get_current_event_time ());
-		gtk_statusbar_push(GTK_STATUSBAR(kz->statusBar), 1, "Gesture:");
+		gtk_statusbar_push(GTK_STATUSBAR(kz->statusBar),
+				   STATUS_GESTURE, "Gesture:");
  	}
 	
 	return FALSE;
 }
 
 static gboolean
-motion_notify_event_cb (GtkWidget *widget, GdkEventMotion *event, KzMozEmbed *kzembed)
+motion_notify_event_cb (GtkWidget *widget,
+			GdkEventMotion *event,
+			KzMozEmbed *kzembed)
 {
+	g_return_val_if_fail(KZ_IS_WINDOW(widget), FALSE);
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
 
 	if (kz_gesture_is_started(kzembed->gesture)) {
@@ -912,9 +1036,8 @@ motion_notify_event_cb (GtkWidget *widget, GdkEventMotion *event, KzMozEmbed *kz
 					    &win_x, &win_y);
 		gdk_window_get_position (GTK_WIDGET(kzembed)->window,
 					 &win_x_pos, &win_y_pos);
-		x = win_x + event->x;
-		y = win_y + event->y;
-
+		gtk_widget_get_pointer (GTK_WIDGET(kzembed),
+					&x, &y);
 		kz_gesture_update_position(kzembed->gesture, x, y);
 	}
 
@@ -922,8 +1045,11 @@ motion_notify_event_cb (GtkWidget *widget, GdkEventMotion *event, KzMozEmbed *kz
 }
 
 static gboolean
-button_release_event_cb (GtkWidget *widget, GdkEventButton *event, KzMozEmbed *kzembed)
+button_release_event_cb (GtkWidget *widget,
+			 GdkEventButton *event,
+			 KzMozEmbed *kzembed)
 {
+	g_return_val_if_fail(KZ_IS_WINDOW(widget), FALSE);
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
 
 	if (kz_gesture_is_started(kzembed->gesture))
@@ -932,9 +1058,10 @@ button_release_event_cb (GtkWidget *widget, GdkEventButton *event, KzMozEmbed *k
 			kz_gesture_perform(kzembed->gesture);
 		else
 			kz_gesture_cancel(kzembed->gesture);
-		gtk_statusbar_push(GTK_STATUSBAR(KZ_WINDOW(widget)->statusBar), 1, "");
+		gtk_statusbar_push(GTK_STATUSBAR(KZ_WINDOW(widget)->statusBar),
+				   STATUS_GESTURE, "");
 	}
-
+	
         if (gdk_pointer_is_grabbed ())
         {
                 gdk_pointer_ungrab (gtk_get_current_event_time ());
@@ -945,55 +1072,3 @@ button_release_event_cb (GtkWidget *widget, GdkEventButton *event, KzMozEmbed *k
 
 	return FALSE;
 }
-
-
-
-
-
-#warning FIXME!!
-static void
-kz_window_go_back (KzWindow *kz)
-{
-	KzMozEmbed *kzembed = KZ_WINDOW_CURRENT_PAGE(kz);
-	if (kzembed)
-		gtk_moz_embed_go_back(GTK_MOZ_EMBED(kzembed));
-}
-static void
-kz_window_go_forward (KzWindow *kz)
-{
-	KzMozEmbed *kzembed = KZ_WINDOW_CURRENT_PAGE(kz);
-	if (kzembed)
-		gtk_moz_embed_go_forward(GTK_MOZ_EMBED(kzembed));
-}
-static void
-kz_window_close_current_tab_idle (KzWindow *kz)
-{
-	KzMozEmbed *kzembed = KZ_WINDOW_CURRENT_PAGE(kz);
-	if (kzembed)
-		kz_window_close_tab_idle(kz, kzembed);
-}
-static void
-kz_window_new_reload (KzWindow *kz)
-{
-	KzMozEmbed *kzembed = KZ_WINDOW_CURRENT_PAGE(kz);
-
-	if (!kzembed) return;
-
-	if (kzembed->gesture_location)
-		kz_window_open_new_tab(kzembed->parent_kazehakase,
-				       kzembed->gesture_location);
-	else
-		gtk_moz_embed_reload(GTK_MOZ_EMBED(kzembed),
-				     GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
-}
-/*
-static void
-kz_window_home (KzWindow *kz)
-{
-	KzMozEmbed *kzembed = KZ_WINDOW_CURRENT_PAGE(kz);
-	if (kzembed) {
-		gtk_moz_embed_load_url(GTK_MOZ_EMBED(kzembed),
-				       prefs.home_uri);
-	}
-}
-*/
