@@ -24,6 +24,10 @@
 #include <gtk/gtk.h>
 #include <gtkmozembed.h>
 
+#include "egg-action-group.h"
+#include "egg-menu-merge.h"
+#include "kz-gesture.h"
+
 G_BEGIN_DECLS
 
 #define KZ_TYPE_WINDOW			(kz_window_get_type ())
@@ -34,10 +38,30 @@ G_BEGIN_DECLS
 #define KZ_WINDOW_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), KZ_TYPE_WINDOW, KzWindowClass))
 
 #define KZ_WINDOW_NTH_PAGE(kz, n) \
-	(KZ_IS_WINDOW(kz) ? KZ_MOZ_EMBED(gtk_notebook_get_nth_page(GTK_NOTEBOOK(KZ_WINDOW(kz)->notebook),n)) : NULL)
+	(KZ_IS_WINDOW(kz) ? gtk_notebook_get_nth_page(GTK_NOTEBOOK(KZ_WINDOW(kz)->notebook),n) : NULL)
 #define KZ_WINDOW_CURRENT_PAGE(kz) \
 	(KZ_IS_WINDOW(kz) ? KZ_WINDOW_NTH_PAGE(kz, gtk_notebook_get_current_page(GTK_NOTEBOOK(kz->notebook))) : NULL)
 
+#define KZ_WINDOW_SET_SENSITIVE(kz, action_name, sensitive) \
+{ \
+	if (KZ_IS_WINDOW(kz) && EGG_IS_ACTION_GROUP(kz->action_group)) { \
+		EggAction *action; \
+		action = egg_action_group_get_action(kz->action_group, \
+						     action_name); \
+		if (action) egg_action_set_sensitive(action, sensitive); \
+	} \
+}
+
+#define KZ_WINDOW_SET_VISIBLE(kz, action_name, visible) \
+{ \
+	if (KZ_IS_WINDOW(kz) && EGG_IS_ACTION_GROUP(kz->action_group)) { \
+		EggAction *action; \
+		action = egg_action_group_get_action(kz->action_group, \
+						     action_name); \
+		if (action) egg_action_set_visible(action, visible); \
+	} \
+}
+
 typedef struct _KzWindow KzWindow;
 typedef struct _KzWindowClass KzWindowClass;
 
@@ -46,18 +70,7 @@ struct _KzWindow
 	GtkWindow   parent;
 
 	GtkWidget  *topLevelVBox;
-	GtkWidget  *menuBar;
-	GtkWidget  *fileMenuItem;
-	GtkWidget  *fileMenu;
-	GtkWidget  *fileClose;
-	GtkWidget  *fileQuit;
-	GtkWidget  *toolbarHBox;
-	GtkWidget  *toolbar;
-	GtkWidget  *backButton;
-	GtkWidget  *stopButton;
-	GtkWidget  *forwardButton;
-	GtkWidget  *reloadButton;
-	GtkWidget  *homeButton;
+
 	GtkWidget  *urlEntry;
 	GtkWidget  *progressAreaHBox;
 	GtkWidget  *progressBar;
@@ -68,12 +81,13 @@ struct _KzWindow
 	GtkWidget  *findArea;
 	GtkWidget  *kztoolBar; /* Kazehakase tool bar */
 
+	GtkAccelGroup  *accel_group;
+	EggActionGroup *action_group;
+	EggMenuMerge   *menu_merge;
+	KzGestureItems *gesture_items;
+	
 	const char *statusMessage;
 	char       *tempMessage;
-	gboolean   menuBarOn;
-	gboolean   toolBarOn;
-	gboolean   locationBarOn;
-	gboolean   statusBarOn;
 	gboolean   didFind; /* for find keyword */
 };
 
@@ -83,9 +97,14 @@ struct _KzWindowClass
 };
 
 GtkType           kz_window_get_type           (void);
+GList            *kz_window_get_window_list    (void);
 GtkWidget        *kz_window_new                (const gchar *url);
 GtkWidget        *kz_window_open_new_tab       (KzWindow *kz,
 						const gchar *url);
+void              kz_window_close_tab          (KzWindow *kz,
+						GtkWidget *widget);
+void              kz_window_close_tab_idle     (KzWindow *kz,
+						GtkWidget *widget);
 void              kz_window_load_url           (KzWindow *kz,
 						const gchar *url);
 void              kz_window_set_visibility     (KzWindow *kz,
